/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright Compaq Computer Corporation, 2001                *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*
 * @(#)$RCSfile: cfgtree.h,v $ $Revision: 1.1.11.1 $ (DEC) $Date: 2000/09/07 18:10:52 $
 */
#ifndef __ARCH_ALPHA_HAL_CFGTREE_H__
#define __ARCH_ALPHA_HAL_CFGTREE_H__

#if defined (__cplusplus)
extern "C" {
#endif /* __cplusplus */
    
struct gct_node {
    unsigned char type;			/* Node type			*/
    unsigned char subtype;		/* Type-specific subtype	*/
    unsigned short int size;		/* Size of node including header*/
    unsigned int  hd_extension;   
    ulong owner;
    ulong current_owner;
    ulong id;				/* Type-specific ID		*/
    union  {
        ulong node_flags;		/* flags			*/
        struct  {
            unsigned node_hardware	: 1; /* Node represents hardware     */
            unsigned node_hotswap	: 1; /* Hardware can be hitswapped   */
            unsigned node_unavailable	: 1; /* HW is not avail (powerdown)  */
            unsigned node_hw_template	: 1; /* Node is a template device    */
            unsigned node_initialized	: 1; /* Partition is initialized     */
            unsigned node_cpu_primary	: 1; /* CPU is a primary             */
            unsigned node_in_console	: 1; /* CPU is in console mode       */
            unsigned node_pwr_down	: 1; /* Node is Powered Down         */
	    unsigned node_pwr_ctrl_point: 1; /* Node can be indep. powered   */
	    unsigned node_present	: 1; /* HW is physically present     */
	    unsigned node_reassignable	: 1; /* HW is reassignable           */	
	    unsigned node_hard_partitioned : 1; /* Pt of hard-partitioning   */
            unsigned node_active	: 1; /* resource is avail for reassign*/
            unsigned node_rsrvd_13_31	: 19; /* Unused bits in first lw     */
            unsigned node_rsrvd_32_63 : 32; /* Unused bits in second lw      */
	} flag_bits;
    } flag_union;
    unsigned short int rev;		/* Revision of this node	     */
    unsigned short int change_counter;  /* Change bit			     */
    unsigned int   rsrvd1;		/* Reserved			     */
    ulong config;			/* Config binging		     */
    ulong affinity;			/* Affinity (performance) binding    */
    long parent;			/* Parent node			     */
    long next_sib;			/* Next sibling node		     */
    long prev_sib;			/* Previous sibling node	     */
    long child;				/* Child node			     */
    ulong fw_usage;			/* SRM/FW usage			     */
    ulong os_usage;			/* OS usage			     */
    ulong reserved3;			/* reserved			     */
    unsigned int checksum;		/* Checksum field		     */
    unsigned int magic;			/* Valid bits 'GLXY'		     */
};

struct gct_root_node {
    struct gct_node hd;
    ulong lock;			/* Software lock			*/
    ulong transient_level;	/* Update counter (in prog)		*/
    ulong current_level;	/* Update counter (actual)		*/
    ulong console_req;		/* Memory required for console		*/
    ulong min_alloc;  		/* Minimum memory allocation		*/
    ulong min_align;		/* Memory allocation alignment		*/
    ulong base_alloc;		/* Base memory min allocation		*/
    ulong base_align; 		/* Base memory alloc alignment		*/
    ulong max_phys_address;	/* Largest Physical Address		*/
    ulong mem_size;		/* Total current memory size		*/
    ulong platform_type;	/* Type code of platform		*/
    ulong platform_name;	/* Offset to name string		*/
    ulong primary_instance;	/* Handle of GALAXY Primary Partiion	*/
    ulong first_free;		/* First free byte of tree pool		*/
    ulong high_limit;		/* High address limit for nodes		*/
    ulong lookaside;		/* Lookaside list for node deletion	*/
    int   available;		/* Amount of bytes in pool		*/
    int   max_partition;	/* Max partitions			*/
    ulong partitions;		/* Offset to partition ID map		*/
    ulong communities;		/* Offset to community ID map		*/
    ulong bindings;		/* Offset to array of bindings		*/
    int   max_platform_partition; /* Max part platform supports 	*/
    int   max_desc;       /* Max memory descriptors			*/
    char  galaxy_id [16];	/* Galaxy ID				*/
    char  galaxy_id_pad [4];	/* Pad ID with a longword of bytes	*/
    ulong root_flags;
};

struct hd_ext {
    unsigned int fru_count;
    unsigned int reserved;
    long fru_handle;
    unsigned int subpkt_count;
    unsigned int subpkt_offset;
};

struct sys_res_config_subpkt_hdr {
    short length;
    short class;
    short type;
    short rev;
    /*
    ulong id;
    int   fru_count;
    int   reserved;
    ulong fru;
    */
    int   diag_flag;
    int   diag_failure_info;
};

/*
 * Type of sys_res_config_subpkt
 */
#define SUBPKT_SENSOR		0x2a

struct return_nodes {
    struct gct_node *node;
    struct return_nodes *next;
};

struct pe_pbm_subpkt {
    struct sys_res_config_subpkt_hdr hdr;
    short reset_configuration;
    short present_0_status;
    short present_1_status;
    short power_control_status;
    short rest_control_status;
    short interrupt_sys_event;
    short hardware_interrupt_ctrl;
    short pio_data_0;
};

struct hose_subpkt {
    struct sys_res_config_subpkt_hdr hdr;
    ulong pca_what_am_i;
    ulong ne_what_am_i;
    ulong fe_what_am_i;
    ulong pbm_pci_present_0;
    ulong pbm_pci_present_1;
};

struct sensor_subpkt {
    struct sys_res_config_subpkt_hdr hdr;
    int	fru_count;
    int	reserved;
    long fru;
    ulong sr_offset;
    int   sensor_class;
    int   properties;
};

struct sysevt_subpkt {
    struct sys_res_config_subpkt_hdr hdr;
    ulong  sysevt_id;
    ulong  sysevt_offset;
};

/* 
 * Sensor subpacket-specific class field values
 */
#define SENSOR_FAN_CLASS	1
#define SENSOR_TEMP_CLASS	2
#define SENSOR_DCPWR_CLASS	4

/* 
 * Sensor subpacket-specific properties field values
 */
#define SENSOR_PROP_TYPE_MASK	0x3
#define SENSOR_PROP_STATUS	0x1
#define SENSOR_PROP_VALUE	0x2
#define SENSOR_PROP_STATUS_AND_VALUE 0x3

static char *node_type[] = {
    "NODE_UNDEFINED_0",
    "NODE_ROOT", 
    "NODE_HW_ROOT", 
    "NODE_SW_ROOT", 
    "NODE_TEMPLATE_ROOT", 
    "NODE_COMMUNITY", 
    "NODE_PARTITION", 
    "NODE_SBB", 
    "NODE_PSEUDO", 
    "NODE_CPU", 
    "NODE_MEMORY_SUB", 
    "NODE_MEMORY_DESC", 
    "NODE_MEMORY_CTRL", 
    "NODE_IOP", 
    "NODE_HOSE", 
    "NODE_BUS", 
    "NODE_IO_CTRL", 
    "NODE_SLOT", 
    "NODE_CPU_MODULE", 
    "NODE_POWER_ENVIR", 
    "NODE_FRU_ROOT", 
    "NODE_FRU_DESC", 
    "NODE_SMB", 
    "NODE_CAB", 
    "NODE_CHASSIS", 
    "NODE_EXP_CHASSIS", 
    "NODE_SYS_INTER_SWITCH", 
    "NODE_HARD_PARTITION", 
    "NODE_RISER",
    "NODE_LAST"
};

static char *node_subtype[] = {
      "SUBNODE_UNKNOWN",
      "SUBNODE_CPU_NOPRIMARY",
      "SUBNODE_PCI",
      "SUBNODE_EISA",
      "SUBNODE_ISA",
      "SUBNODE_XMI",
      "SUBNODE_FBUS",
      "SUBNODE_XBUS",
      "SUBNODE_USB",
      "SUBNODE_SERIAL_PORT",
      "SUBNODE_FLOPPY",
      "SUBNODE_PARALLEL_PORT",
      "SUBNODE_SCSI",
      "SUBNODE_IDE",
      "SUBNODE_NI",
      "SUBNODE_FDDI",
      "SUBNODE_TOKEN_RING",
      "SUBNODE_NI_SCSI",
      "SUBNODE_GRAPHICS",
      "SUBNODE_ATM",
      "SUBNODE_MEM_CHAN",
      "SUBNODE_CI",
      "SUBNODE_1394",
      "SUBNODE_AGP",
      "SUBNODE_SUPER_HIPPI",
      "SUBNODE_FIBERCHANNEL",
      "SUBNODE_CAB",
      "SUBNODE_CHASSIS",
      "SUBNODE_EXP_CHASSIS",
      "SUBNODE_POWER_SUPPLY",
      "SUBNODE_COOLING",
      "SUBNODE_SIMM",
      "SUBNODE_DIMM",
      "SUBNODE_RIMM",
      "SUBNODE_CPU_MODULE",
      "SUBNODE_CPU_CACHE_MODULE",
      "SUBNODE_CPU_MEMORY_MODULE",
      "SUBNODE_MEMORY_CARRIER_MODULE",
      "SUBNODE_BACKPLANE_ASSY",
      "SUBNODE_MOTHER_BOARD",
      "SUBNODE_FAN",
      "SUBNODE_SYS_INTERCONN_PORT",
      "SUBNODE_SYS_INTERCONN_SWITCH",
      "SUBNODE_MEMORY_COHERENCY_MODULE",
      "SUBNODE_IO_PORT_MODULE",
      "SUBNODE_REMOTE_MGNT_MODULE",
      "SUBNODE_POWER_ENV_CTRL_MODULE",
      "SUBNODE_MULTI_DEVICE",
      "SUBNODE_IO_EXP_CAB",
      "SUBNODE_EXPANSION_CAB",
      "SUBNODE_EXP_CHASSIS",
      "SUBNODE_DAUGHTER_CARD",
      "SUBNODE_RISER_CARD",
      "SUBNODE_CPU_FAN",
      "SUBNODE_MASTER_CLOCK",
      "SUBNODE_CLOCK_DIST",
      "SUBNODE_LAST"
};

#ifndef _KERNEL
static char *subpkt_type[] = {
	"SUBPKT_UNDEFINED_0",
	"SUBPKT_PLATFORM", 
	"SUBPKT_PROCESSOR",
	"SUBPKT_MEM",
	"SUBPKT_SYS_BUS_BRIDGE",
	"SUBPKT_PCI_DEV",
	"SUBPKT_EISA_DEV",
	"SUBPKT_CACHE",
	"SUBPKT_POWER",
	"SUBPKT_COOLING",
	"SUBPKT_SYS_INIT_LOG",
	"SUBPKT_XMI_DEV",
	"SUBPKT_FBUS_DEV",
	"SUBPKT_VME",
	"SUBPKT_SBB",
	"SUBPKT_IOP",
	"SUBPKT_HOSE",
	"SUBPKT_BUS",
	"SUBPKT_ISA_DEV",
	"SUBPKT_USB_DEV",
	"SUBPKT_CONSOLE",
	"SUBPKT_PWR_ENV_SUMMARY",
	"SUBPKT_UNDEFINED_22",
	"SUBPKT_UNDEFINED_23",
	"SUBPKT_UNDEFINED_24",
	"SUBPKT_UNDEFINED_25",
	"SUBPKT_UNDEFINED_26",
	"SUBPKT_UNDEFINED_27",
	"SUBPKT_UNDEFINED_28",
	"SUBPKT_UNDEFINED_29",
	"SUBPKT_PCI_VPD",
	"SUBPKT_SMB",
	"SUBPKT_FC",
	"SUBPKT_AGP",
	"SUBPKT_IDE",
	"SUBPKT_SCSI",
	"SUBPKT_1394",
	"SUBPKT_SHPPI",
	"SUBPKT_MEM_DIR",
	"SUBPKT_NUMA_PORT",
	"SUBPKT_NUMA_SWITCH",
	"SUBPKT_RMC",
	"SUBPKT_SENSOR",
	"SUBPKT_SYSEVENT",
	"SUBPKT_UNDEFINED_44",
	"SUBPKT_UNDEFINED_45",
	"SUBPKT_UNDEFINED_46",
	"SUBPKT_UNDEFINED_47",
	"SUBPKT_UNDEFINED_48",
	"SUBPKT_UNDEFINED_49"
};

static char *sensor_prop[] = {
	"SENSOR_PROP_UNDEFINED_0",
	"status",			/* 1 */
	"value",			/* 2 */
	"status & value",		/* 3 */
	"writeable",			/* 4 */
	"writeable, status",		/* 5 */
	"writeable, value",		/* 6 */
	"writeable, status & value", 	/* 7 */
	"UpperLong Support",		/* 8 */
	"UpperLong, status",		/* 9 */
	"UpperLong, value",		/* 10 */
	"UpperLong, status, value",	/* 11 */
	"UpperLong, writeable",		/* 12 */
	"UpperLong, writeable, status",	/* 13 */
	"UpperLong, writeable, value",	/* 14 */
	"UpperLong, writeable, value, status",	/* 15 */
	"SENSOR_PROP_UNDEFINED_16",
	"SENSOR_PROP_UNDEFINED_17",
	"SENSOR_PROP_UNDEFINED_18",
	"SENSOR_PROP_UNDEFINED_19"
};

static char *sensor_class[] = {
	"SENSOR_CLASS_UNDEFINED_0",
	"FAN",
	"TEMP",
	"AC PWR",
	"DC PWR",
	"BATT PWR"
};
#endif	/* !_KERNEL */

/* Define NODE Type */
#define CFGTREE_NODE_ROOT               1
#define CFGTREE_NODE_HW_ROOT            2
#define CFGTREE_NODE_SW_ROOT            3
#define CFGTREE_NODE_TEMPLATE_ROOT      4
#define CFGTREE_NODE_COMMUNITY          5
#define CFGTREE_NODE_PARTITION          6
#define CFGTREE_NODE_SBB                7
#define CFGTREE_NODE_PSEUDO             8
#define CFGTREE_NODE_CPU                9
#define CFGTREE_NODE_MEMORY_SUB        10
#define CFGTREE_NODE_MEMORY_DESC       11
#define CFGTREE_NODE_MEMORY_CTRL       12
#define CFGTREE_NODE_IOP               13
#define CFGTREE_NODE_HOSE              14
#define CFGTREE_NODE_BUS               15
#define CFGTREE_NODE_IO_CTRL           16
#define CFGTREE_NODE_SLOT              17
#define CFGTREE_NODE_CPU_MODULE        18
#define CFGTREE_NODE_POWER_ENVIR       19
#define CFGTREE_NODE_FRU_ROOT          20
#define CFGTREE_NODE_FRU_DESC          21
#define CFGTREE_NODE_SMB               22
#define CFGTREE_NODE_CAB               23
#define CFGTREE_NODE_CHASSIS           24
#define CFGTREE_NODE_EXP_CHASSIS       25
#define CFGTREE_NODE_SYS_INTER_SWITCH  26
#define CFGTREE_NODE_HARD_PARTITION    27
#define CFGTREE_NODE_RISER             28
#define CFGTREE_NODE_LAST              29


/* Define NODE SubTypes */
#define CFGTREE_SUBNODE_UNKNOWN			  0
#define CFGTREE_SUBNODE_CPU_NOPRIMARY		  1
#define CFGTREE_SUBNODE_PCI			  2
#define CFGTREE_SUBNODE_EISA			  3
#define CFGTREE_SUBNODE_ISA			  4
#define CFGTREE_SUBNODE_XMI			  5
#define CFGTREE_SUBNODE_FBUS			  6
#define CFGTREE_SUBNODE_XBUS			  7
#define CFGTREE_SUBNODE_USB			  8
#define CFGTREE_SUBNODE_SERIAL_PORT		  9
#define CFGTREE_SUBNODE_FLOPPY			 10 
#define CFGTREE_SUBNODE_PARALLEL_PORT		 11 
#define CFGTREE_SUBNODE_SCSI			 12 
#define CFGTREE_SUBNODE_IDE			 13 
#define CFGTREE_SUBNODE_NI			 14 
#define CFGTREE_SUBNODE_FDDI			 15 
#define CFGTREE_SUBNODE_TOKEN_RING		 16 
#define CFGTREE_SUBNODE_NI_SCSI			 17 
#define CFGTREE_SUBNODE_GRAPHICS		 18 
#define CFGTREE_SUBNODE_ATM			 19 
#define CFGTREE_SUBNODE_MEM_CHAN		 20 
#define CFGTREE_SUBNODE_CI			 21 
#define CFGTREE_SUBNODE_1394			 22 
#define CFGTREE_SUBNODE_AGP			 23 
#define CFGTREE_SUBNODE_SUPER_HIPPI		 24 
#define CFGTREE_SUBNODE_FIBERCHANNEL		 25 
#define CFGTREE_SUBNODE_CAB			 26 
#define CFGTREE_SUBNODE_CHASSIS			 27 
#define CFGTREE_SUBNODE_EXP_CHASSIS		 28 
#define CFGTREE_SUBNODE_POWER_SUPPLY		 29 
#define CFGTREE_SUBNODE_COOLING			 30 
#define CFGTREE_SUBNODE_SIMM			 31 
#define CFGTREE_SUBNODE_DIMM			 32 
#define CFGTREE_SUBNODE_RIMM			 33 
#define CFGTREE_SUBNODE_CPU_MODULE		 34 
#define CFGTREE_SUBNODE_CPU_CACHE_MODULE	 35 
#define CFGTREE_SUBNODE_CPU_MEMORY_MODULE	 36 
#define CFGTREE_SUBNODE_MEMORY_CARRIER_MODULE    37 
#define CFGTREE_SUBNODE_BACKPLANE_ASSY		 38 
#define CFGTREE_SUBNODE_MOTHER_BOARD		 39 
#define CFGTREE_SUBNODE_FAN			 40 
#define CFGTREE_SUBNODE_SYS_INTERCONN_PORT	 41 
#define CFGTREE_SUBNODE_SYS_INTERCONN_SWITCH	 42 
#define CFGTREE_SUBNODE_MEMORY_COHERENCY_MODULE  43 
#define CFGTREE_SUBNODE_IO_PORT_MODULE		 44 
#define CFGTREE_SUBNODE_REMOTE_MGNT_MODULE 	 45 
#define CFGTREE_SUBNODE_POWER_ENV_CTRL_MODULE    46 
#define CFGTREE_SUBNODE_MULTI_DEVICE		 47 
#define CFGTREE_SUBNODE_IO_EXP_CAB		 48 
#define CFGTREE_SUBNODE_EXPANSION_CAB		 49 
#define CFGTREE_SUBNODE_RESERVED                 50 
#define CFGTREE_SUBNODE_DAUGHTER_CARD            51 
#define CFGTREE_SUBNODE_RISER_CARD               52 
#define CFGTREE_SUBNODE_CPU_FAN                  53 
#define CFGTREE_SUBNODE_MASTER_CLOCK             54 
#define CFGTREE_SUBNODE_CLOCK_DIST               55 
#define CFGTREE_SUBNODE_POWER_SUB_RACK           56
#define CFGTREE_SUBNODE_POWER_CABINET            57
#define CFGTREE_SUBNODE_SYSTEM_CABINET           58
#define CFGTREE_SUBNODE_VOLTAGE_REGULATOR_MODULE 59
#define CFGTREE_SUBNODE_CPU_CHIP                 60
#define CFGTREE_SUBNODE_IO_EXPANSION_DRAWER      61
#define CFGTREE_SUBNODE_SYSTEM_DRAWER            62
#define CFGTREE_SUBNODE_PCI_X                    63
#define CFGTREE_SUBNODE_IO_EXP_ASSEMBLY          64
#define CFGTREE_SUBNODE_LAST			 65

/* Define Tree Search-by Types */
#define CFGTREE_SEARCH_BY_TYPE           0
#define CFGTREE_SEARCH_BY_ID             1
#define CFGTREE_SEARCH_BY_TYPE_AND_ID    2

/* Soft QBB ID starts at bit 48 in the NODE ID */
#define CFGTREE_QBB_ID_SHIFT 48
#define CFGTREE_QBB_ID_MASK  0x00ff000000000000
    
/* Hose Number starts at bit in the NODE ID */
#define CFGTREE_HOSE_ID_SHIFT       0
#define CFGTREE_HOSE_AND_QBB_ID_MASK 0x00ff0000000000ff
    
/* Function Prototypes */
struct gct_node *gct_search_tree(ulong attr0, 
				 ulong attr1, 
				 ulong mask, 
				 long offset, 
				 int query_type);
struct gct_node *gct_get_child(struct gct_node *node);
struct gct_node *gct_get_sibling(struct gct_node *node);
struct gct_node *gct_get_parent(struct gct_node *node);
struct sys_res_config_subpkt_hdr *
		 gct_get_subpkt(struct gct_node *node, 
				struct sys_res_config_subpkt_hdr *subpkt);
int              gct_sysevt_get_sr_offset(struct gct_node *node);
void             gct_print_node(struct gct_node *node);

#if defined (__cplusplus)
}
#endif /* __cplusplus */

#endif	/* __ARCH_ALPHA_HAL_CFGTREE_H__ */
