/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright Compaq Computer Corporation, 2000                *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*
 * @(#)$RCSfile: vm_umap.h,v $ $Revision: 1.1.24.3 $ (DEC) $Date: 1999/07/29 20:14:38 $
 */
#ifndef	__VM_UMAP__
#define	__VM_UMAP__ 1
#include <kern/lock.h>
#include <sys/unix_defs.h>
#include <vm/vm_vlock.h>
#include <sys/param.h>
#include <mach/vm_param.h>
#include <sys/mman.h>

/*
 * user mode private map structure
 * The following members can change after creation, and are protected
 * by the map lock:
 * 
 * 	um_actual_size
 * 	um_vpage
 * 	um_flags
 * 	um_vlock
 */

struct u_map_private {
	vm_size_t		um_maxvas;	/* Maximum VAS allowed */
	vm_size_t		um_maxwired;	/* Max wired space allowed */
	vm_size_t		um_wired;	/* Pages wired */
						/* 
						 * For a working set model.
						 * Any other policy leads
						 * to complication when
						 * maintaining these fields.
						 */
	vm_size_t		um_maxrss;	/* Maximum rss allowed */
	vm_size_t		um_rss;		/* Resident pages in map */
	vm_size_t		um_actual_size;	/* actual size */
	int			um_vpage;	/* vpages allocated */
	unsigned int		:1,		/* Formerly, um_lock_future */
				um_unload_all:1, /* unloading all address sp */
						 /* Restore um_unload_all to */
						 /* its original position so */
						 /* TCR 1.6 which uses this */
						 /* field can find it without */
						 /* have to recompile, etc. */
				:30;
	unsigned long		um_flags;
	struct vm_vlock		*um_vlock;	/* Virtual space lock by K */
};
/*
 * Bit masks for um_flags field
 */
#define UM_LOCK_FUTURE_READ	0x1		/* lock future R pages */
#define UM_LOCK_FUTURE_WRITE	0x2		/* lock future W pages */
#define UM_LOCK_FUTURE_EXEC	0x4		/* lock future E pages */

/*
 * The code that makes use of these bits assume
 * that the user-level API bit definitions match
 * those defined in the u_map_private structure.
 * The follow conditionals verify that the
 * definitions are consistent.
 */

#if UM_LOCK_FUTURE_READ != PROT_READ
#error	"UM_LOCK_FUTURE_READ != PROT_READ, check definitions"
#endif	/* UM_LOCK_FUTURE_READ != PROT_READ */

#if UM_LOCK_FUTURE_WRITE != PROT_WRITE
#error	"UM_LOCK_FUTURE_WRITE != PROT_WRITE, check definitions"
#endif	/* UM_LOCK_FUTURE_WRITE != PROT_WRITE */

#if UM_LOCK_FUTURE_EXEC != PROT_EXEC
#error	"UM_LOCK_FUTURE_EXEC != PROT_EXEC, check definitions"
#endif	/* UM_LOCK_FUTURE_EXEC != PROT_EXEC */

#define UM_LOCK_FUTURE		(UM_LOCK_FUTURE_READ | UM_LOCK_FUTURE_WRITE \
				| UM_LOCK_FUTURE_EXEC)
#define UM_LOCK_FUTURE_TEXT	(UM_LOCK_FUTURE_READ | UM_LOCK_FUTURE_EXEC)
#define UM_LOCK_FUTURE_DATA	(UM_LOCK_FUTURE_READ | UM_LOCK_FUTURE_WRITE)

#define	vm_mape_lockaddr(VME) simple_lock_addr((VME)->vme_faultlock)

/*
 * The vm_mape_faultwait is called by a thread that has the address map
 * read or write locked.  The thread goes to sleep on the fault lock until
 * vme_faults count goes to zero.
 */
#define vm_mape_faultwait(VME) {					\
	if ((VME)->vme_faults) {					\
		simple_lock(&(VME)->vme_faultlock);			\
		if ((VME)->vme_faults) {				\
			do {						\
				(VME)->vme_faultwait = 1;		\
				assert_wait_mesg((vm_offset_t)(VME),	\
						  FALSE, "fault");	\
				simple_unlock(&(VME)->vme_faultlock);	\
				thread_block();				\
				simple_lock(&(VME)->vme_faultlock);	\
			} while ((VME)->vme_faults);			\
		}							\
		simple_unlock(&(VME)->vme_faultlock);			\
	}								\
}

/*
 * Described return states from u_map_is_wired, see function for definitions.
 */
typedef enum { UMAP_NOCHG, UMAP_KWIRED, UMAP_UWIRED, UMAP_NOWIRED } u_mapwire_t;

#ifdef	KERNEL
extern boolean_t
	lock_future(vm_map_entry_t, vm_map_t),
	u_map_entry_delete_check(vm_map_t map, vm_map_entry_t ep, 
			vm_offset_t start, vm_offset_t end),
	u_map_entry_clip_check(vm_map_t map, vm_map_entry_t ep,
			vm_offset_t start, vm_offset_t end);

u_mapwire_t u_map_is_wired(vm_map_entry_t, vm_offset_t, vm_offset_t, vm_prot_t);
#endif	/* KERNEL */

#endif /* !__VM_UMAP__ */
